# Select Edges in Profiles
# Date - Monday, July 16, 2012
# Version 1.0.1
# Main thing on how to locate the edges in profiles is by TIG
# Written by Anton Synytsia
# Not a plugin, but written for simple use of testing

# Menu Location: UI_menu:plugins/selectEdgesInProfiles

# Usage: Select grp(s) or/and component instance(s) and then select the menu item

# Change Log:
#   Fixed the selection back edges.

# Yet, Unsolved bugs:
#   Selects edges that camera doesn't sees.
#   TIG.getEdges function doesn't yet search the group for edges found inside the parent group.

module TIG
  def self.getEdges(entity) #entity represents group or component
    edges = Array.new
    entity.entities.each{|ent|
      edges.push(ent) if (ent.class == Sketchup::Edge)
    }
    return (edges.uniq)
  end #getEdges

  def self.getEdgesInProfile(parent_ent) #parent_ent must be a group or a component
    return false if (parent_ent.class != Sketchup::Group) and (parent_ent.class != Sketchup::ComponentInstance)
    model = Sketchup.active_model
    view = model.active_view
    camera = view.camera
    edges_in_profile = Array.new
    getEdges(parent_ent).each{|edge|
      efaces = edge.faces.to_a
      if (efaces.length != 2) #if true - the edge is in profile, otherwise continue.
        edges_in_profile.push(edge)
        next
      end
      estart = edge.start.position.transform(parent_ent.transformation) # edge.start current position
      vector = estart.vector_to(camera.eye)
      #vector = camera.eye.vector_to(estart) # try inverted... fun
      count = 0
      efaces.each{|face|
        fnormal = face.normal.transform(parent_ent.transformation) #face.normal relative to global axis
        theta = fnormal.angle_between(vector)
        count+=1 if (theta >= 90.degrees)
      }
      puts count
      edges_in_profile.push(edge) if (count == 1)
    }
    # select the edges in profiles for testing
    Sketchup.active_model.selection.add(edges_in_profile)
    return edges_in_profile
  end #getEdgesInProfile
end #TIG

unless file_loaded?(__FILE__)
  menu = UI.menu("plugins")
  menu.add_item("SelectEdgesInProfiles"){
    selection = Sketchup.active_model.selection.to_a
    Sketchup.active_model.selection.clear
    selection.each{|ent| TIG.getEdgesInProfile(ent)}
  }
  file_loaded(__FILE__)
end